// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef OKULAR_VERSION_H
#define OKULAR_VERSION_H

#define OKULAR_VERSION_STRING "23.08.2"
#define OKULAR_VERSION_MAJOR 23
#define OKULAR_VERSION_MINOR 8
#define OKULAR_VERSION_PATCH 2
#define OKULAR_VERSION ((23<<16)|(8<<8)|(2))

#endif
